# Script name: Simulation of a population using A+D model
#
# Authors: Jon Bancic, Daniel Tolhurst, Chris Gaynor, Gregor Gorjanc
#
# Date Created: 2024-01-23
#
# This script provides an example of simulating a population 
# under additive + dominance model without and with AlphaSimR.

rm(list=ls())
library(package="Matrix")
library(package="AlphaSimR")

#-- Simulation parameters
nInd = 1000              # number of individuals
nQtl = 100               # number of QTLs affecting the trait
nMarkers = 900           # number of markers
nLoci = nQtl + nMarkers  # number of segregating sites
mean = 5                 # overall trait mean
varA = 3                 # additive variance
meanDD = 0.92            # mean dominance deviation
varDD  = 0.20            # variance of dominance deviations
h2   = 0.4               # narrow-sense heritability


#-- Simulate genotype data
geno = matrix(nrow = nInd, ncol = nLoci)
alleleFreq = runif(n = nLoci, min = 0.01, max = 0.99)
for (i in 1:nLoci) {
  geno[, i] = rbinom(n = nInd, size = 2, prob = alleleFreq[i])-1
}
colnames(geno) = paste0("site",1:nLoci)
rownames(geno)  = paste0("id",1:nInd)
# Check
dim(geno)
geno[1:10,1:10]
hist(alleleFreq)
cor(colMeans(geno)/2, alleleFreq)


#-- Sample QTL positions
posQtl = sample(size = nQtl, 1:nLoci, replace = F)
posQtl = posQtl[order(posQtl)]
posSnp = setdiff(1:nLoci, posQtl)
# No overlap between SNPs and QTLs
sum(posQtl %in% posSnp)


#-- Sample additive effects
addEff = rnorm(nQtl, mean = 0, sd = 1)


#-- Sample dominance effects
# Provide mean and variance of dominance deviations
domEff <- rnorm(nQtl, mean = meanDD, sd = sqrt(varDD))
# Multiply with absolute additive effects
domEff <- abs(addEff) * domEff
# Create dominance matrix
geno_d <- abs(abs(geno) - 1)


#-- Calculate genetic values
# Additive values
av = geno[, posQtl, drop = FALSE] %*% addEff
# Dominance values
dv = geno_d[, posQtl, drop = FALSE] %*% domEff
# Genetic values
gv = av + dv


# Scale effects to obtain correct variances
# Option 1: scale according to additive variance
# scale  = sqrt(varA)/sqrt(var(av)) 
# Option 2: scale according to total genetic variance
scale  = sqrt(varA)/sqrt(var(gv))
addEff = addEff * rep(scale, nQtl)
domEff = domEff * rep(scale, nQtl)
intercept = mean - mean(gv * rep(scale, nQtl))
av = geno[, posQtl, drop = FALSE] %*% addEff
dv = geno_d[, posQtl, drop = FALSE] %*% domEff
gv = av + dv
# Check that mean and variance are correct
intercept + mean(gv)
var(av)
var(dv)
var(gv)


#-- Simulate noise according to h2 specified
varE = var(av)*(1-h2)/h2
e = rnorm(n = nInd, sd = sqrt(varE))


#-- Obtain phenotypes
pheno = intercept + gv + e
colnames(pheno) = "Trait 1"

# Check mean, variance and heritabilty 
mean(pheno)
var(pheno)
cor(pheno, gv)^2


#-- Construct a population
pop = list(id     = rownames(geno), 
           gv     = gv, 
           pheno  = pheno,
           loci   = colnames(geno),
           posQtl = colnames(geno)[posQtl],
           posSnp = colnames(geno)[posSnp],
           geno   = geno,
           addEff = addEff
)

str(pop)

#===============================================================
#-- Now perform the same operation with AlphaSimR!

#-- Create founder haplotypes
founderPop = quickHaplo(nInd = nInd,
                        nChr = 1,
                        segSites = nLoci)

#-- Set simulation parameters
SP = SimParam$new(founderPop)

#-- Add additive trait
SP$addTraitAD(nQtlPerChr = nQtl,
              mean = mean,
              var  = varA, 
              meanDD = meanDD,
              varDD = varDD, 
              useVarA = T)

#-- Create population
pop2 = newPop(founderPop)
str(pop2)

#-- Set pheno
pop2 = setPheno(pop2, h2 = h2)

#-- Check that population parameters are correct
hist(apply(pullSegSiteGeno(pop2), 2, function(X) mean(X)/2)) # allele freqs
meanG(pop2)
varG(pop2)
meanP(pop2)
varP(pop2)
cor(pop2@gv,pop2@pheno)^2

#-- Let's check linkage disequilibrium
LD = cor(pullQtlGeno(pop2))
hist(LD[upper.tri(LD)])
image(Matrix(LD))
LD = LD[upper.tri(LD)]
posQtl = getQtlMap(trait = 1)[,"site"]
dist = outer(posQtl, posQtl, FUN = "-")
dist = dist[upper.tri(dist)]
smoothScatter(x = -dist, y = abs(LD))
lines(lowess(x = -dist, y = abs(LD)), col = "red")
